@@ -1,7 +1,7 @@
 (ns puppetlabs.puppetdb.mq
   (:import [org.apache.activemq.broker BrokerService]
            [org.apache.activemq ScheduledMessage]
-           [org.apache.activemq.usage SystemUsage]
+           [org.apache.activemq.usage SystemUsage MemoryUsage]
            [javax.jms Connection Message TextMessage BytesMessage Session]
            [org.apache.activemq ActiveMQConnectionFactory]
            [org.apache.activemq.pool PooledConnectionFactory])
@@ -10,7 +10,15 @@
             [puppetlabs.puppetdb.utils :as utils]
             [puppetlabs.puppetdb.schema :refer [defn-validated]]
             [schema.core :as s]
-            [slingshot.slingshot :refer [throw+]]))
+            [slingshot.slingshot :refer [throw+]]
+            [metrics.timers :refer [timer time!]]
+            [puppetlabs.puppetdb.metrics.core :as metrics]))
+
+(def mq-metrics-registry (get-in metrics/metrics-registries [:mq :registry]))
+
+(def metrics (atom {:message-persistence-time (timer mq-metrics-registry
+                                                     (metrics/keyword->metric-name
+                                                       [:global] :message-persistence-time))}))
 
 (defn- set-usage!*
   "Internal helper function for setting `SystemUsage` values on a `BrokerService`
@@ -46,6 +54,17 @@
   [broker megabytes]
   (set-usage!* broker megabytes #(.getStoreUsage %) "StoreUsage"))
 
+(defn- set-memory-usage!
+  "Configures the `MemoryUsage` setting for an instance of `BrokerService`.
+
+  `broker`     - the `BrokerService` to configure
+  `megabytes ` - the maximum amount of memory usage to allow for the
+  BrokerService, or `nil` to use the default value of 1GB.
+
+  Returns the (potentially modified) `broker` object."
+  [broker megabytes]
+  (set-usage!* broker megabytes #(.getMemoryUsage %) "MemoryUsage"))
+
 (defn- set-temp-usage!
   "Configures the `TempUsage` setting for an instance of `BrokerService`.
 
@@ -103,6 +122,7 @@
               (.setSchedulerSupport true)
               (.setPersistent true)
               (enable-jmx true)
+              (set-memory-usage! (:memory-usage config))
               (set-store-usage! (:store-usage config))
               (set-temp-usage!  (:temp-usage config)))
          mc (doto (.getManagementContext mq)
@@ -240,10 +260,11 @@
   ([connection endpoint message]
    (send-message! connection endpoint message {}))
   ([connection endpoint message attributes]
-   (with-open [s (.createSession connection true 0)
-               pro (.createProducer s (.createQueue s endpoint))]
-     (commit-or-rollback s
-       (.send pro (to-jms-message s message attributes))))))
+   (time! (get @metrics :message-persistence-time)
+          (with-open [s (.createSession connection true 0)
+                      pro (.createProducer s (.createQueue s endpoint))]
+            (commit-or-rollback s
+                                (.send pro (to-jms-message s message attributes)))))))
 
 (defn-validated timed-drain-into-vec! :- [jms-message-schema]
   "Drains messages from the indicated endpoint into a vector via connection.
@@ -280,17 +301,13 @@
   (str "org.apache.activemq:type=Broker,brokerName=" name))
 
 (defn-validated queue-size
-  "Returns the number of pending messages in the named broker queue.
-  Throws {:type ::queue-not-found :broker broker :name name} when the
-  queue doesn't exist."
-  [broker :- s/Str
-   queue :- s/Str]
+  "Returns the number of pending messages in the queue.
+  Throws {:type ::queue-not-found} when the queue doesn't exist."
+  []
   (try
-    (jmx/read (format "%s,destinationType=Queue,destinationName=%s"
-                      (broker-mb broker) queue)
-              :QueueSize)
+    (jmx/read "puppetlabs.puppetdb.mq:name=global.depth" :Count)
     (catch javax.management.InstanceNotFoundException ex
-      (throw+ {:type ::queue-not-found :broker broker :name queue}))))
+      (throw+ {:type ::queue-not-found}))))
 
 (defn-validated transfer-messages!
   "Transfers all of the messages currently available in the named
